# Generated by Django 2.2.3 on 2019-08-11 12:17

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('avishan', '0002_activationcode_user_group'),
    ]

    operations = [
        migrations.CreateModel(
            name='ExceptionRecord',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('date_created', models.DateTimeField(auto_now_add=True)),
                ('class_title', models.CharField(max_length=255)),
                ('url', models.CharField(max_length=255)),
                ('http_method', models.CharField(max_length=255)),
            ],
            options={
                'abstract': False,
            },
        ),
    ]
