PROJECT_NAME = ''
MONITORED_URLS_START = ['/api/v1/']
MONITORED_ANONYMOUS_URLS = ['/api/v1/auth/check_phone/', '/api/v1/auth/check_code/', ]
NOT_MONITORED_URLS = []
BASE_URLS = []

JWT_KEY = '*'

USER_RESPONSE_DICT_FIELDS = ['phone', 'first_name', 'last_name']

RECOMMEND_CODE_CHOICES = 'ABCDEFGHJKLMNPQRSTUWXYZ23456789'

SMS_CODE_LENGTH = 4
SMS_CODE_VALID_SECONDS = 4 * 60
SMS_RESEND_ALLOWED_SECONDS = 90
SMS_SIGNIN_TEMPLATE = '*'
SMS_SIGNUP_TEMPLATE = '*'
KAVENEGAR_API_TOKEN = '*'
RECOMMEND_CODE_FROM_USER = 'profile__recommend_code'
AUTO_CONVERT_TO_BCHDATETIME = True

CHABOK_ACCESS_TOKEN = ''

HTTP_METHODS = ['GET', 'POST', 'PUT', 'DELETE', 'OPTIONS']