# Generated by Django 2.2 on 2019-09-02 18:01

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('avishan', '0005_auto_20190818_0336'),
    ]

    operations = [
        migrations.AddField(
            model_name='exceptionrecord',
            name='exception_args',
            field=models.TextField(null=True),
        ),
        migrations.AddField(
            model_name='exceptionrecord',
            name='request_headers',
            field=models.TextField(null=True),
        ),
    ]
