# Generated by Django 2.2 on 2019-09-03 11:54

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('avishan', '0006_auto_20190902_1801'),
    ]

    operations = [
        migrations.AddField(
            model_name='exceptionrecord',
            name='checked',
            field=models.BooleanField(default=False),
        ),
    ]
