# Generated by Django 2.2 on 2019-09-11 19:29

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('avishan', '0008_auto_20190905_1748'),
    ]

    operations = [
        migrations.DeleteModel(
            name='Animal',
        ),
        migrations.DeleteModel(
            name='Place',
        ),
        migrations.DeleteModel(
            name='Room',
        ),
    ]
