# Generated by Django 2.2.5 on 2019-09-15 10:44

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('avishan', '0010_auto_20190913_1817'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='exceptionrecord',
            name='datetime',
        ),
    ]
