# Generated by Django 2.2.3 on 2019-08-02 15:19

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('avishan', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='activationcode',
            name='user_group',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='avishan.UserGroup'),
        ),
    ]
